@@ -9,8 +9,8 @@ from uno import createUnoStruct
 from unogenerator.reusing.listdict_functions import listdict_min
 from time import sleep
 
-__version__ = '0.21.0'
-__versiondatetime__=datetime(2022, 3, 9, 12, 8)
+__version__ = '0.23.0'
+__versiondatetime__=datetime(2022, 3, 22, 9, 44)
 __versiondate__=__versiondatetime__.date()
 
 try:
@@ -114,6 +114,18 @@ def index2column(index):
 class Coord:
     def __init__(self, strcoord):
         self.letter, self.number=self.__extract(strcoord)
+        
+        
+    ## Creates a Coord object from spreadsheet index coords
+    @classmethod
+    def from_index(cls, column_index, row_index):
+        return cls(index2column(column_index)+index2row(row_index))
+        
+            
+    ## Creates a Coord object from spreadsheet letters
+    @classmethod
+    def from_letters(cls, column, letter):
+        return cls(column+letter)
 
     def __repr__(self):
         return f"Coord <{self}>"
@@ -220,10 +232,45 @@ class Range:
         
     @classmethod
     def from_coords_indexes(cls, start_letter_index, start_number_index, end_letter_index, end_number_index):
-        c_start=Coord_from_index(start_letter_index, start_number_index)
-        c_end=Coord_from_index(end_letter_index, end_number_index)
+        c_start=Coord.from_index(start_letter_index, start_number_index)
+        c_end=Coord.from_index(end_letter_index, end_number_index)
         return cls(f"{c_start}:{c_end}")
 
+    ## Creates a Range object from itself stard and end coords
+    @classmethod
+    def from_coords(cls, start, end):
+        start=Coord.assertCoord(start)
+        end=Coord.assertCoord(end)
+        return cls(f"{start}:{end}")
+    
+    ## Creates a Range object from Changing the start_coord of a range
+    @classmethod
+    def from_start_coord_change(cls, range, new_start_coord):
+        return cls.from_columns_rows(new_start_coord, range.numColumns(), range.numRows())
+        
+    ## Creates a Range object from a coord_start and the number of columms and rows of therange
+    @classmethod
+    def from_columns_rows(cls, coord_start, number_columns,  number_rows):
+        start=Coord.assertCoord(coord_start)
+        end=Coord(start).addRow(number_rows-1).addColumn(number_columns-1)
+        return cls(f"{start}:{end}")
+
+    @classmethod
+    def from_iterable_object(cls,  coord_start, o):
+        coord_start=Coord.assertCoord(coord_start)
+        r=Range(f"{coord_start.string()}:{coord_start.string()}")
+
+        if len(o)==0:
+            return r
+
+        if o[0].__class__.__name__ in ("list","dict", "OrderedDict"): #Iterables con len
+            len_rows=len(o)-1
+            len_columns=len(o[0])-1
+        else:
+            len_rows=0
+            len_columns=len(o)-1
+        return Range(f"{coord_start.string()}:{coord_start.addRowCopy(len_rows).addColumnCopy(len_columns).string()}")
+
     ##Return the outcome of the test b in a. Note the reversed operands.
     def __contains__(self, b):
         if (    b.letterIndex()>=self.start.letterIndex() and 
@@ -233,15 +280,7 @@ class Range:
             return True
         return False
 
-    ## Returns a list of rows of all Coord objects in the range
-    def coords(self):
-        r=[]
-        for row in range(self.numRows()):
-            tmprow=[]
-            for column in range(self.numColumns()):
-                tmprow.append(self.start.addRowCopy(row).addColumnCopy(column))
-            r.append(tmprow)
-        return r
+
 
     ## Converts a string to a Range. Returns None if conversion can't be done
     def __extract(self,range):
@@ -326,17 +365,18 @@ class Range:
                     row.append((letter_index, number_index))
                 r.append(row)
         return r
-
+        
+    ## Returns a list of rows of all Coord objects in the range
     def coords_list(self, plain=False):
         r=[]
         if plain is True:
             for letter_index, number_index in self.indexes_list():
-                r.append(Coord_from_index(letter_index, number_index))
+                r.append(Coord.from_index(letter_index, number_index))
         else:
             for row  in self.indexes_list():
                 r2=[]
                 for  letter_index , number_index  in row:
-                    r2.append(Coord_from_index(letter_index, number_index))
+                    r2.append(Coord.from_index(letter_index, number_index))
                 r.append(r2)
         return r
 
@@ -360,19 +400,7 @@ def addDebugSystem(level):
 
 
 
-## Creates a Coord object from spreadsheet letters
-def Coord_from_letters(column, letter):
-    return Coord(column+letter)
 
-## Creates a Coord object from spreadsheet index coords
-def Coord_from_index(column_index, row_index):
-    return Coord(index2column(column_index)+index2row(row_index))
-    
-## Creates a Range object from itself stard and end coords
-def Range_from_coords(start, end):
-    start=Coord.assertCoord(start)
-    end=Coord.assertCoord(end)
-    return Range(f"{start}:{end}")
 
 def generate_formula_total_string(key, coord_from, coord_to):
     if key == "#SUM":
@@ -431,20 +459,6 @@ def next_port(last,  first_port,  instances):
     else:
         return last+1
 
-def get_range_from_iterable_object( coord_start, o):
-    coord_start=Coord.assertCoord(coord_start)
-    r=Range(f"{coord_start.string()}:{coord_start.string()}")
-
-    if len(o)==0:
-        return r
-
-    if o[0].__class__.__name__ in ("list","dict", "OrderedDict"): #Iterables con len
-        len_rows=len(o)-1
-        len_columns=len(o[0])-1
-    else:
-        len_rows=0
-        len_columns=len(o)-1
-    return Range(f"{coord_start.string()}:{coord_start.addRowCopy(len_rows).addColumnCopy(len_columns).string()}")
 
 ## @param attempts (Integer). Sometimes when server is busy this method fails to detect info. So I make several attempts with a time interval
 ## @return listdict with process info
