@@ -8,9 +8,6 @@ from com.sun.star.beans import PropertyValue
 from com.sun.star.text import ControlCharacter
 from com.sun.star.awt import Size
 from com.sun.star.sheet.ConditionEntryType import COLORSCALE
-
-
-
 from com.sun.star.style.ParagraphAdjust import RIGHT,  LEFT
 from com.sun.star.style.BreakType import PAGE_BEFORE, PAGE_AFTER
 from gettext import translation
@@ -18,7 +15,8 @@ from logging import warning, debug
 from pkg_resources import resource_filename
 from shutil import copyfile
 from tempfile import TemporaryDirectory
-from unogenerator.commons import Coord as C, ColorsNamed,  Range as R, datetime2uno, guess_object_style, row2index, column2index, datetime2localc1989, date2localc1989,  time2localc1989, Coord_from_letters, Coord_from_index, next_port, get_from_process_numinstances_and_firstport,  is_formula
+from unogenerator.commons import Coord as C, ColorsNamed,  Range as R, datetime2uno, guess_object_style, row2index, column2index, datetime2localc1989, date2localc1989,  time2localc1989, next_port, get_from_process_numinstances_and_firstport,  is_formula
+from unogenerator.reusing.casts import lor_transposed
 from unogenerator.reusing.currency import Currency
 from unogenerator.reusing.datetime_functions import string2dtnaive, string2date, string2time
 from unogenerator.reusing.percentage import Percentage
@@ -77,7 +75,17 @@ class ODF:
                 print(_(f"Changing port {old} to {self.loserver_port} {i} times"))
                 if i==maxtries - 1:
                     print(_("This process died"))
-
+        
+    ## This method allows to use with statement. 
+    ## with ODS() as doc:
+    ##      doc.createSheet("WITH")
+    ## First calls __init__ with None and 2002, then enter, then exit
+    def __enter__(self):
+        return self
+        
+    ## Exit function to use with with statement. __enter__ defines enter in with
+    def __exit__(self, *args, **kwargs):
+        self.close()
 
     def calculateAll(self):
         self.document.calculateAll()
@@ -108,7 +116,6 @@ class ODF:
         
 
     def setLanguage(self, language, country):
-        print("FALTA")
         self.language="es"
         self.country="ES"
 
@@ -404,7 +411,7 @@ class ODS(ODF):
     def __init__(self, template=None, loserver_port=2002):
         ODF.__init__(self, template, loserver_port)
         self._remove_default_sheet=True
-        
+
     def getRemoveDefaultSheet(self):
         return self._remove_default_sheet
         
@@ -518,16 +525,18 @@ class ODS(ODF):
     def addListOfRowsWithStyle(self, coord_start, list_rows, colors=ColorsNamed.White, styles=None, cellbycell=False):
         coord_start=C.assertCoord(coord_start) 
         
+        rows=len(list_rows)
+        if rows==0:
+            columns=0
+        else:
+            columns=len(list_rows[0])
+        
         if cellbycell is True:
             for i, row in enumerate(list_rows):
                 self.addRowWithStyle(coord_start.addRowCopy(i), row, colors=colors,styles=styles)
+                return R.from_columns_rows(coord_start, columns, rows)
         else:
             #Calculates the number of rows and columns of list_rows
-            rows=len(list_rows)
-            if rows==0:
-                columns=0
-            else:
-                columns=len(list_rows[0])
 
             #Sets colors and styles
             if colors.__class__.__name__!="list" and columns>0:
@@ -550,7 +559,7 @@ class ODS(ODF):
                 r.append(r_row)
 
             #Writes data fast
-            range_indexes=(coord_start.letterIndex(), coord_start.numberIndex(), coord_start.letterIndex()+columns-1, coord_start.numberIndex()+rows-1)
+            range_indexes=[coord_start.letterIndex(), coord_start.numberIndex(), coord_start.letterIndex()+columns-1, coord_start.numberIndex()+rows-1]
             range=self.sheet.getCellRangeByPosition(coord_start.letterIndex(), coord_start.numberIndex(), coord_start.letterIndex()+columns-1, coord_start.numberIndex()+rows-1)
             range.setDataArray(r)
 
@@ -563,11 +572,24 @@ class ODS(ODF):
             return R.from_coords_indexes(*range_indexes)
 
     ## @param style If None tries to guess it
-    def addListOfColumnsWithStyle(self, coord_start, list_columns, colors=ColorsNamed.White, styles=None):
-        coord_start=C.assertCoord(coord_start)
-        for i, column in enumerate(list_columns):
-            self.addColumnWithStyle(coord_start.addColumnCopy(i), column, colors=colors,styles=styles)
-
+    def addListOfColumnsWithStyle(self, coord_start, list_columns, colors=ColorsNamed.White, styles=None, cellbycell=False):
+        coord_start=C.assertCoord(coord_start) 
+        
+        if cellbycell is True:
+            if len(list_columns)>0:
+                columns=len(list_columns)
+                rows=len(list_columns[0])
+            else:
+                columns=0
+                rows=0
+            
+            for i, column in enumerate(list_columns):
+                self.addColumnWithStyle(coord_start.addColumnCopy(i), column, colors=colors,styles=styles)
+                return R.from_columns_rows(coord_start, columns, rows)
+        else:
+            list_rows=lor_transposed(list_columns)
+            return self.addListOfRowsWithStyle(coord_start, list_rows, colors, styles, cellbycell)
+            
     ## Making prints in each statement we can see that most of the time is interactuating with libreoffice server (Times in a very fast machine)    ##A4999 Time after assert 0:00:00.000002
     ## Thats why we create adding all celss with range.setDataArray in addListOfRowsWithStyle
     ##A4999 Time after guessing style 0:00:00.000001
@@ -702,7 +724,7 @@ class ODS(ODF):
         self.document.getCurrentController().freezeAtPosition(freeze.letterIndex(), freeze.numberIndex())
 
         if selected is None:
-            selected=Coord_from_index(num_columns-1, num_rows-1)
+            selected=C.from_index(num_columns-1, num_rows-1)
         else:
             selected=C.assertCoord(selected)
         selectedcell=self.sheet.getCellByPosition(selected.letterIndex(), selected.numberIndex())
@@ -719,7 +741,7 @@ class ODS(ODF):
                 number=freeze.number
             else:
                 number=minus_coord.number
-            topleft=Coord_from_letters(letter, number)
+            topleft=C.from_letters(letter, number)
         else:
             topleft=C.assertCoord(topleft)
         self.document.getCurrentController().setFirstVisibleColumn(topleft.letterIndex())
@@ -736,16 +758,18 @@ class ODS(ODF):
         self.statistics.appendCellGetValuesStartMoment(start)
         return r
 
-
     ## Returns a list of rows with the values of the sheet
     ## @param sheet_index Integer index of the sheet
     ## @param skip_up int. Number of rows to skip at the begining of the list of rows (lor)
     ## @param skip_down int. Number of rows to skip at the end of the list of rows (lor)
     ## @return Returns a list of rows of object values
     def getValues(self, skip_up=0, skip_down=0,  detailed=False):
-        if skip_up>0 or skip_down>0:
-            print("FALTA, REMOVE FROM RANGE")
-        return self.getValuesByRange(self.getSheetRange(), detailed)
+        range=self.getSheetRange()
+        if skip_up>0:
+           range=range.addRowBefore(-skip_up)
+        if skip_down>0:
+           range=range.addRowAfter(-skip_down)
+        return self.getValuesByRange(range, detailed)
 
     ## @param sheet_index Integer index of the sheet
     ## @param range_ Range object to get values. If None returns all values from sheet
